import os
import numpy as np
import nibabel as nib
from src.nexi.estimate_nexi_noiseless import estimate_nexi_noiseless


def test_estimate_nexi_noiseless():
    np.random.seed(123)
    if not os.path.isdir('tests/nexi/data/nexi'):
        os.mkdir('tests/nexi/data/nexi')
    estimate_nexi_noiseless('tests/nexi/data/phantom.nii.gz',
                            'tests/nexi/data/phantom.bval', 'tests/nexi/data/phantom.td',
                            'tests/nexi/data/nexi', mask_path='tests/nexi/data/mask.nii.gz',
                            debug=False)

    powder_average_filename = 'tests/nexi/data/nexi/powderaverage_dwi.nii.gz'
    bval_filename = 'tests/nexi/data/nexi/powderaverage.bval'
    td_filename = 'tests/nexi/data/nexi/powderaverage.td'
    powder_average_npz_filename = 'tests/nexi/data/nexi/powderaverage_signal.npz'

    assert np.allclose(nib.load(powder_average_filename).get_fdata(),
                       nib.load('tests/nexi/data/powderaverage_dwi_ref.nii.gz').get_fdata(), equal_nan=True)
    assert np.allclose(np.loadtxt(bval_filename), np.loadtxt('tests/nexi/data/powderaverage_ref.bval'))
    assert np.allclose(np.loadtxt(td_filename), np.loadtxt('tests/nexi/data/powderaverage_ref.td'))

    os.remove(powder_average_filename)
    os.remove(bval_filename)
    os.remove(td_filename)
    os.remove(powder_average_npz_filename)

    parameters = ["t_ex", "Di", "De", "f"]
    for param in parameters:
        param_filename = f'tests/nexi/data/nexi/nexi_{param}.nii.gz'
        param_ref_filename = f'tests/nexi/data/models_ref/nexi_{param}_ref.nii.gz'
        assert np.allclose(nib.load(param_filename).get_fdata(),
                           nib.load(param_ref_filename).get_fdata(), equal_nan=True)
        os.remove(param_filename)

    os.rmdir('tests/nexi/data/nexi')



import numpy as np
from abc import ABC


class MIcroSTructureParameters(ABC):
    """Microstructure parameters."""

    def __init__(self, mist_model, param=None):
        """Initialize the microstructure parameters."""
        # number of parameters
        self.n_params = mist_model.n_params
        # parameter names
        self.names = mist_model.param_names
        # parameters
        self.param = param

    def initialize_fixed_parameters(self, n_set, ground_truth_params):
        # parameters
        param = np.empty([n_set, self.n_params])
        for param_index in range(self.n_params):
            param[:, param_index] = np.ones(n_set) * ground_truth_params[param_index]
        self.param = param
        return param

    def initialize_random_parameters(self, n_set, microstruct_model):
        # parameters
        param = np.empty([n_set, self.n_params])
        for param_index in range(microstruct_model.n_params):
            param[:, param_index] = np.random.rand(n_set) * \
                                    (microstruct_model.param_lim[param_index][1] - microstruct_model.param_lim[param_index][0]) \
                                    + microstruct_model.param_lim[param_index][0]
        if microstruct_model.constraints is not None:
            for constr in microstruct_model.constraints:
                param = constr(param)
        self.param = param
        return param


class MIcroSTructureParametersException(Exception):
    """Handle exceptions related to microstructure parameters."""
    pass


class InvalidMIcroSTructureParameters(MIcroSTructureParametersException):
    """Handle exceptions related to wrong microstructure parameters."""
    pass



# SPDX-FileCopyrightText: 2023-present Quentin Uhl <quentin.uhl@gmail.com>
#
# SPDX-License-Identifier: Apache 2.0

from .estimate_nexi_noiseless import estimate_nexi_noiseless
from .estimate_nexi import estimate_nexi
from .powderaverage.powderaverage import powder_average, save_data_as_npz, normalize_sigma
